



if not exist %PREFIX%\\Library\\bin\\aws-c-common.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\include\\aws\\common\\config.h exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\lib\\cmake\\AwsCFlags.cmake exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
